combine_draws combines and resamples parameter draws returned from an MCMC algorithm.
combine_draws(draws, r)A matrix or data frame containing `R` randomly sampled rows from the combined `betadraw` components.
A list of draws from a posterior predictive distribution
Number of MCMC draws
Federico Bumbaca, Leeds School of Business, University of Colorado Boulder, federico.bumbaca@colorado.edu
Bumbaca, F. (Rico), Misra, S., & Rossi, P. E. (2020). Scalable Target Marketing: Distributed Markov Chain Monte Carlo for Bayesian Hierarchical Models. Journal of Marketing Research, 57(6), 999-1018.
rhierLinearMixtureParallel,
rhierMnlRwMixtureParallel,
rhierLinearDPParallel,
rhierMnlDPParallel